// avg_seq_list.h
#ifndef H_ASL
#define H_ASL

#include <stdlib.h>

const size_t NODES_PER_ALLOC = 100;

typedef struct node {
  union {
    float value;
    struct node *ptr;
  } v;
  struct node *next;
} _node;

typedef struct {
  _node *head;
  struct node *available_node_memory;
  int remaining_availible_nodes;
  struct node* free_list;
} avg_seq_list;

static avg_seq_list *asl_create() {
  // use calloc to return zeroed memory--we'll use a NULL
  // head pointer to denote an empty list.
  avg_seq_list * list = (avg_seq_list *) calloc(1, sizeof(avg_seq_list));
  list->remaining_availible_nodes = NODES_PER_ALLOC;
  list->available_node_memory = (_node *) calloc(NODES_PER_ALLOC, sizeof(_node));
  list->head = nullptr;
  list->free_list = nullptr;

  return list;
}

static void asl_append(avg_seq_list *asl, float x) {

  if (asl->remaining_availible_nodes <= 0) {
    _node *nd = (_node *) calloc(1, sizeof(_node));
    nd->v.ptr = asl->available_node_memory;
    nd->next = asl->free_list;
    asl->free_list = nd;

    asl->available_node_memory = (_node*) calloc(NODES_PER_ALLOC, sizeof(_node));
    asl->remaining_availible_nodes = NODES_PER_ALLOC;
  }

  _node *tmp = asl->available_node_memory + --(asl->remaining_availible_nodes);

  tmp->v.value = x;
  tmp->next = asl->head;
  asl->head = tmp;
}

static float asl_average(avg_seq_list *asl) {
  float total = 0;
  size_t cnt = 0;
  for (_node *nd=asl->head; nd; nd=nd->next) {
    total += nd->v.value;
    cnt++;
  }

  return total / (float) cnt;
}

static size_t asl_storage(avg_seq_list *asl) {
  size_t total = 0; // we won't count the asl type in the total
                    // as it isn't really required.
  for (_node *nd=asl->head; nd; nd=nd->next) {
    total += sizeof(*nd);
  }

  return total;
}

static void asl_destroy(avg_seq_list *asl) {
  _node *head = asl->free_list;
  while (head) {
    auto next = head->next;
    free(head->v.ptr);
    free(head);
    head = next;
  }

  free(asl->available_node_memory);
  free(asl);
}
#endif
